/* spencermountain/spacetime-daylight 1.3.1 MIT */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.spacetime = {}));
}(this, (function (exports) { 'use strict';

	function commonjsRequire () {
		throw new Error('Dynamic requires are not currently supported by rollup-plugin-commonjs');
	}

	function createCommonjsModule(fn, module) {
		return module = { exports: {} }, fn(module, module.exports), module.exports;
	}

	var suncalc = createCommonjsModule(function (module, exports) {
	  /*
	   (c) 2011-2015, Vladimir Agafonkin
	   SunCalc is a JavaScript library for calculating sun/moon position and light phases.
	   https://github.com/mourner/suncalc
	  */
	  (function () {

	    var PI = Math.PI,
	        sin = Math.sin,
	        cos = Math.cos,
	        tan = Math.tan,
	        asin = Math.asin,
	        atan = Math.atan2,
	        acos = Math.acos,
	        rad = PI / 180; // sun calculations are based on http://aa.quae.nl/en/reken/zonpositie.html formulas
	    // date/time constants and conversions

	    var dayMs = 1000 * 60 * 60 * 24,
	        J1970 = 2440588,
	        J2000 = 2451545;

	    function toJulian(date) {
	      return date.valueOf() / dayMs - 0.5 + J1970;
	    }

	    function fromJulian(j) {
	      return new Date((j + 0.5 - J1970) * dayMs);
	    }

	    function toDays(date) {
	      return toJulian(date) - J2000;
	    } // general calculations for position


	    var e = rad * 23.4397; // obliquity of the Earth

	    function rightAscension(l, b) {
	      return atan(sin(l) * cos(e) - tan(b) * sin(e), cos(l));
	    }

	    function declination(l, b) {
	      return asin(sin(b) * cos(e) + cos(b) * sin(e) * sin(l));
	    }

	    function azimuth(H, phi, dec) {
	      return atan(sin(H), cos(H) * sin(phi) - tan(dec) * cos(phi));
	    }

	    function altitude(H, phi, dec) {
	      return asin(sin(phi) * sin(dec) + cos(phi) * cos(dec) * cos(H));
	    }

	    function siderealTime(d, lw) {
	      return rad * (280.16 + 360.9856235 * d) - lw;
	    }

	    function astroRefraction(h) {
	      if (h < 0) // the following formula works for positive altitudes only.
	        h = 0; // if h = -0.08901179 a div/0 would occur.
	      // formula 16.4 of "Astronomical Algorithms" 2nd edition by Jean Meeus (Willmann-Bell, Richmond) 1998.
	      // 1.02 / tan(h + 10.26 / (h + 5.10)) h in degrees, result in arc minutes -> converted to rad:

	      return 0.0002967 / Math.tan(h + 0.00312536 / (h + 0.08901179));
	    } // general sun calculations


	    function solarMeanAnomaly(d) {
	      return rad * (357.5291 + 0.98560028 * d);
	    }

	    function eclipticLongitude(M) {
	      var C = rad * (1.9148 * sin(M) + 0.02 * sin(2 * M) + 0.0003 * sin(3 * M)),
	          // equation of center
	      P = rad * 102.9372; // perihelion of the Earth

	      return M + C + P + PI;
	    }

	    function sunCoords(d) {
	      var M = solarMeanAnomaly(d),
	          L = eclipticLongitude(M);
	      return {
	        dec: declination(L, 0),
	        ra: rightAscension(L, 0)
	      };
	    }

	    var SunCalc = {}; // calculates sun position for a given date and latitude/longitude

	    SunCalc.getPosition = function (date, lat, lng) {
	      var lw = rad * -lng,
	          phi = rad * lat,
	          d = toDays(date),
	          c = sunCoords(d),
	          H = siderealTime(d, lw) - c.ra;
	      return {
	        azimuth: azimuth(H, phi, c.dec),
	        altitude: altitude(H, phi, c.dec)
	      };
	    }; // sun times configuration (angle, morning name, evening name)


	    var times = SunCalc.times = [[-0.833, 'sunrise', 'sunset'], [-0.3, 'sunriseEnd', 'sunsetStart'], [-6, 'dawn', 'dusk'], [-12, 'nauticalDawn', 'nauticalDusk'], [-18, 'nightEnd', 'night'], [6, 'goldenHourEnd', 'goldenHour']]; // adds a custom time to the times config

	    SunCalc.addTime = function (angle, riseName, setName) {
	      times.push([angle, riseName, setName]);
	    }; // calculations for sun times


	    var J0 = 0.0009;

	    function julianCycle(d, lw) {
	      return Math.round(d - J0 - lw / (2 * PI));
	    }

	    function approxTransit(Ht, lw, n) {
	      return J0 + (Ht + lw) / (2 * PI) + n;
	    }

	    function solarTransitJ(ds, M, L) {
	      return J2000 + ds + 0.0053 * sin(M) - 0.0069 * sin(2 * L);
	    }

	    function hourAngle(h, phi, d) {
	      return acos((sin(h) - sin(phi) * sin(d)) / (cos(phi) * cos(d)));
	    } // returns set time for the given sun altitude


	    function getSetJ(h, lw, phi, dec, n, M, L) {
	      var w = hourAngle(h, phi, dec),
	          a = approxTransit(w, lw, n);
	      return solarTransitJ(a, M, L);
	    } // calculates sun times for a given date and latitude/longitude


	    SunCalc.getTimes = function (date, lat, lng) {
	      var lw = rad * -lng,
	          phi = rad * lat,
	          d = toDays(date),
	          n = julianCycle(d, lw),
	          ds = approxTransit(0, lw, n),
	          M = solarMeanAnomaly(ds),
	          L = eclipticLongitude(M),
	          dec = declination(L, 0),
	          Jnoon = solarTransitJ(ds, M, L),
	          i,
	          len,
	          time,
	          Jset,
	          Jrise;
	      var result = {
	        solarNoon: fromJulian(Jnoon),
	        nadir: fromJulian(Jnoon - 0.5)
	      };

	      for (i = 0, len = times.length; i < len; i += 1) {
	        time = times[i];
	        Jset = getSetJ(time[0] * rad, lw, phi, dec, n, M, L);
	        Jrise = Jnoon - (Jset - Jnoon);
	        result[time[1]] = fromJulian(Jrise);
	        result[time[2]] = fromJulian(Jset);
	      }

	      return result;
	    }; // moon calculations, based on http://aa.quae.nl/en/reken/hemelpositie.html formulas


	    function moonCoords(d) {
	      // geocentric ecliptic coordinates of the moon
	      var L = rad * (218.316 + 13.176396 * d),
	          // ecliptic longitude
	      M = rad * (134.963 + 13.064993 * d),
	          // mean anomaly
	      F = rad * (93.272 + 13.229350 * d),
	          // mean distance
	      l = L + rad * 6.289 * sin(M),
	          // longitude
	      b = rad * 5.128 * sin(F),
	          // latitude
	      dt = 385001 - 20905 * cos(M); // distance to the moon in km

	      return {
	        ra: rightAscension(l, b),
	        dec: declination(l, b),
	        dist: dt
	      };
	    }

	    SunCalc.getMoonPosition = function (date, lat, lng) {
	      var lw = rad * -lng,
	          phi = rad * lat,
	          d = toDays(date),
	          c = moonCoords(d),
	          H = siderealTime(d, lw) - c.ra,
	          h = altitude(H, phi, c.dec),
	          // formula 14.1 of "Astronomical Algorithms" 2nd edition by Jean Meeus (Willmann-Bell, Richmond) 1998.
	      pa = atan(sin(H), tan(phi) * cos(c.dec) - sin(c.dec) * cos(H));
	      h = h + astroRefraction(h); // altitude correction for refraction

	      return {
	        azimuth: azimuth(H, phi, c.dec),
	        altitude: h,
	        distance: c.dist,
	        parallacticAngle: pa
	      };
	    }; // calculations for illumination parameters of the moon,
	    // based on http://idlastro.gsfc.nasa.gov/ftp/pro/astro/mphase.pro formulas and
	    // Chapter 48 of "Astronomical Algorithms" 2nd edition by Jean Meeus (Willmann-Bell, Richmond) 1998.


	    SunCalc.getMoonIllumination = function (date) {
	      var d = toDays(date || new Date()),
	          s = sunCoords(d),
	          m = moonCoords(d),
	          sdist = 149598000,
	          // distance from Earth to Sun in km
	      phi = acos(sin(s.dec) * sin(m.dec) + cos(s.dec) * cos(m.dec) * cos(s.ra - m.ra)),
	          inc = atan(sdist * sin(phi), m.dist - sdist * cos(phi)),
	          angle = atan(cos(s.dec) * sin(s.ra - m.ra), sin(s.dec) * cos(m.dec) - cos(s.dec) * sin(m.dec) * cos(s.ra - m.ra));
	      return {
	        fraction: (1 + cos(inc)) / 2,
	        phase: 0.5 + 0.5 * inc * (angle < 0 ? -1 : 1) / Math.PI,
	        angle: angle
	      };
	    };

	    function hoursLater(date, h) {
	      return new Date(date.valueOf() + h * dayMs / 24);
	    } // calculations for moon rise/set times are based on http://www.stargazing.net/kepler/moonrise.html article


	    SunCalc.getMoonTimes = function (date, lat, lng, inUTC) {
	      var t = new Date(date);
	      if (inUTC) t.setUTCHours(0, 0, 0, 0);else t.setHours(0, 0, 0, 0);
	      var hc = 0.133 * rad,
	          h0 = SunCalc.getMoonPosition(t, lat, lng).altitude - hc,
	          h1,
	          h2,
	          rise,
	          set,
	          a,
	          b,
	          xe,
	          ye,
	          d,
	          roots,
	          x1,
	          x2,
	          dx; // go in 2-hour chunks, each time seeing if a 3-point quadratic curve crosses zero (which means rise or set)

	      for (var i = 1; i <= 24; i += 2) {
	        h1 = SunCalc.getMoonPosition(hoursLater(t, i), lat, lng).altitude - hc;
	        h2 = SunCalc.getMoonPosition(hoursLater(t, i + 1), lat, lng).altitude - hc;
	        a = (h0 + h2) / 2 - h1;
	        b = (h2 - h0) / 2;
	        xe = -b / (2 * a);
	        ye = (a * xe + b) * xe + h1;
	        d = b * b - 4 * a * h1;
	        roots = 0;

	        if (d >= 0) {
	          dx = Math.sqrt(d) / (Math.abs(a) * 2);
	          x1 = xe - dx;
	          x2 = xe + dx;
	          if (Math.abs(x1) <= 1) roots++;
	          if (Math.abs(x2) <= 1) roots++;
	          if (x1 < -1) x1 = x2;
	        }

	        if (roots === 1) {
	          if (h0 < 0) rise = i + x1;else set = i + x1;
	        } else if (roots === 2) {
	          rise = i + (ye < 0 ? x2 : x1);
	          set = i + (ye < 0 ? x1 : x2);
	        }

	        if (rise && set) break;
	        h0 = h2;
	      }

	      var result = {};
	      if (rise) result.rise = hoursLater(t, rise);
	      if (set) result.set = hoursLater(t, set);
	      if (!rise && !set) result[ye > 0 ? 'alwaysUp' : 'alwaysDown'] = true;
	      return result;
	    }; // export as Node module / AMD module / browser variable


	    module.exports = SunCalc;
	  })();
	});

	var spacetimeGeo = createCommonjsModule(function (module, exports) {
	  /* spacetime-geo v1.2.0
	     github.com/spencermountain/spacetime-geo
	     MIT
	  */
	  (function (f) {
	    {
	      module.exports = f();
	    }
	  })(function () {
	    return function () {
	      function r(e, n, t) {
	        function o(i, f) {
	          if (!n[i]) {
	            if (!e[i]) {
	              var c = "function" == typeof commonjsRequire && commonjsRequire;
	              if (!f && c) return c(i, !0);
	              if (u) return u(i, !0);
	              var a = new Error("Cannot find module '" + i + "'");
	              throw a.code = "MODULE_NOT_FOUND", a;
	            }

	            var p = n[i] = {
	              exports: {}
	            };
	            e[i][0].call(p.exports, function (r) {
	              var n = e[i][1][r];
	              return o(n || r);
	            }, p, p.exports, r, e, n, t);
	          }

	          return n[i].exports;
	        }

	        for (var u = "function" == typeof commonjsRequire && commonjsRequire, i = 0; i < t.length; i++) {
	          o(t[i]);
	        }

	        return o;
	      }

	      return r;
	    }()({
	      1: [function (_dereq_, module, exports) {
	        function tzlookup(Y, W) {

	          var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
	              U = ["Africa/Abidjan", "Africa/Accra", "Africa/Addis_Ababa", "Africa/Algiers", "Africa/Asmara", "Africa/Bamako", "Africa/Bangui", "Africa/Banjul", "Africa/Bissau", "Africa/Blantyre", "Africa/Brazzaville", "Africa/Bujumbura", "Africa/Cairo", "Africa/Casablanca", "Africa/Ceuta", "Africa/Conakry", "Africa/Dakar", "Africa/Dar_es_Salaam", "Africa/Djibouti", "Africa/Douala", "Africa/El_Aaiun", "Africa/Freetown", "Africa/Gaborone", "Africa/Harare", "Africa/Johannesburg", "Africa/Juba", "Africa/Kampala", "Africa/Khartoum", "Africa/Kigali", "Africa/Kinshasa", "Africa/Lagos", "Africa/Libreville", "Africa/Lome", "Africa/Luanda", "Africa/Lubumbashi", "Africa/Lusaka", "Africa/Malabo", "Africa/Maputo", "Africa/Maseru", "Africa/Mbabane", "Africa/Mogadishu", "Africa/Monrovia", "Africa/Nairobi", "Africa/Ndjamena", "Africa/Niamey", "Africa/Nouakchott", "Africa/Ouagadougou", "Africa/Porto-Novo", "Africa/Sao_Tome", "Africa/Tripoli", "Africa/Tunis", "Africa/Windhoek", "America/Adak", "America/Anchorage", "America/Anguilla", "America/Antigua", "America/Araguaina", "America/Argentina/Buenos_Aires", "America/Argentina/Catamarca", "America/Argentina/Cordoba", "America/Argentina/Jujuy", "America/Argentina/La_Rioja", "America/Argentina/Mendoza", "America/Argentina/Rio_Gallegos", "America/Argentina/Salta", "America/Argentina/San_Juan", "America/Argentina/San_Luis", "America/Argentina/Tucuman", "America/Argentina/Ushuaia", "America/Aruba", "America/Asuncion", "America/Atikokan", "America/Bahia", "America/Bahia_Banderas", "America/Barbados", "America/Belem", "America/Belize", "America/Blanc-Sablon", "America/Boa_Vista", "America/Bogota", "America/Boise", "America/Cambridge_Bay", "America/Campo_Grande", "America/Cancun", "America/Caracas", "America/Cayenne", "America/Cayman", "America/Chicago", "America/Chihuahua", "America/Costa_Rica", "America/Creston", "America/Cuiaba", "America/Curacao", "America/Danmarkshavn", "America/Dawson_Creek", "America/Denver", "America/Detroit", "America/Dominica", "America/Edmonton", "America/Eirunepe", "America/El_Salvador", "America/Fort_Nelson", "America/Fortaleza", "America/Glace_Bay", "America/Godthab", "America/Goose_Bay", "America/Grand_Turk", "America/Grenada", "America/Guadeloupe", "America/Guatemala", "America/Guayaquil", "America/Guyana", "America/Halifax", "America/Havana", "America/Hermosillo", "America/Indiana/Indianapolis", "America/Indiana/Knox", "America/Indiana/Marengo", "America/Indiana/Petersburg", "America/Indiana/Tell_City", "America/Indiana/Vincennes", "America/Iqaluit", "America/Jamaica", "America/Juneau", "America/Kentucky/Louisville", "America/Kentucky/Monticello", "America/Kralendijk", "America/La_Paz", "America/Lima", "America/Los_Angeles", "America/Lower_Princes", "America/Maceio", "America/Managua", "America/Manaus", "America/Marigot", "America/Martinique", "America/Matamoros", "America/Mazatlan", "America/Menominee", "America/Merida", "America/Mexico_City", "America/Miquelon", "America/Moncton", "America/Monterrey", "America/Montevideo", "America/Montserrat", "America/Nassau", "America/New_York", "America/Nome", "America/Noronha", "America/North_Dakota/Beulah", "America/North_Dakota/New_Salem", "America/Ojinaga", "America/Panama", "America/Pangnirtung", "America/Paramaribo", "America/Phoenix", "America/Port-au-Prince", "America/Port_of_Spain", "America/Porto_Velho", "America/Puerto_Rico", "America/Punta_Arenas", "America/Rankin_Inlet", "America/Recife", "America/Regina", "America/Rio_Branco", "America/Santarem", "America/Santiago", "America/Santo_Domingo", "America/Sao_Paulo", "America/Scoresbysund", "America/Sitka", "America/St_Barthelemy", "America/St_Johns", "America/St_Kitts", "America/St_Lucia", "America/St_Thomas", "America/St_Vincent", "America/Tegucigalpa", "America/Thule", "America/Thunder_Bay", "America/Tijuana", "America/Toronto", "America/Tortola", "America/Vancouver", "America/Whitehorse", "America/Winnipeg", "America/Yakutat", "America/Yellowknife", "Antarctica/Casey", "Antarctica/Davis", "Antarctica/DumontDUrville", "Antarctica/Macquarie", "Antarctica/Mawson", "Antarctica/McMurdo", "Antarctica/Rothera", "Antarctica/Syowa", "Antarctica/Troll", "Antarctica/Vostok", "Arctic/Longyearbyen", "Asia/Aden", "Asia/Almaty", "Asia/Amman", "Asia/Anadyr", "Asia/Aqtau", "Asia/Aqtobe", "Asia/Ashgabat", "Asia/Atyrau", "Asia/Baghdad", "Asia/Bahrain", "Asia/Baku", "Asia/Bangkok", "Asia/Barnaul", "Asia/Beirut", "Asia/Bishkek", "Asia/Brunei", "Asia/Chita", "Asia/Choibalsan", "Asia/Colombo", "Asia/Damascus", "Asia/Dhaka", "Asia/Dili", "Asia/Dubai", "Asia/Dushanbe", "Asia/Famagusta", "Asia/Gaza", "Asia/Hebron", "Asia/Ho_Chi_Minh", "Asia/Hong_Kong", "Asia/Hovd", "Asia/Irkutsk", "Asia/Jakarta", "Asia/Jayapura", "Asia/Jerusalem", "Asia/Kabul", "Asia/Kamchatka", "Asia/Karachi", "Asia/Kathmandu", "Asia/Khandyga", "Asia/Kolkata", "Asia/Krasnoyarsk", "Asia/Kuala_Lumpur", "Asia/Kuching", "Asia/Kuwait", "Asia/Macau", "Asia/Magadan", "Asia/Makassar", "Asia/Manila", "Asia/Muscat", "Asia/Nicosia", "Asia/Novokuznetsk", "Asia/Novosibirsk", "Asia/Omsk", "Asia/Oral", "Asia/Phnom_Penh", "Asia/Pontianak", "Asia/Pyongyang", "Asia/Qatar", "Asia/Qostanay", "Asia/Qyzylorda", "Asia/Riyadh", "Asia/Sakhalin", "Asia/Samarkand", "Asia/Seoul", "Asia/Shanghai", "Asia/Singapore", "Asia/Srednekolymsk", "Asia/Taipei", "Asia/Tashkent", "Asia/Tbilisi", "Asia/Tehran", "Asia/Thimphu", "Asia/Tokyo", "Asia/Tomsk", "Asia/Ulaanbaatar", "Asia/Urumqi", "Asia/Ust-Nera", "Asia/Vientiane", "Asia/Vladivostok", "Asia/Yakutsk", "Asia/Yangon", "Asia/Yekaterinburg", "Asia/Yerevan", "Atlantic/Azores", "Atlantic/Bermuda", "Atlantic/Canary", "Atlantic/Cape_Verde", "Atlantic/Faroe", "Atlantic/Madeira", "Atlantic/Reykjavik", "Atlantic/South_Georgia", "Atlantic/St_Helena", "Atlantic/Stanley", "Australia/Adelaide", "Australia/Brisbane", "Australia/Broken_Hill", "Australia/Currie", "Australia/Darwin", "Australia/Eucla", "Australia/Hobart", "Australia/Lord_Howe", "Australia/Melbourne", "Australia/Perth", "Australia/Sydney", "Etc/GMT", "Etc/GMT+1", "Etc/GMT+10", "Etc/GMT+11", "Etc/GMT+12", "Etc/GMT+2", "Etc/GMT+3", "Etc/GMT+4", "Etc/GMT+5", "Etc/GMT+6", "Etc/GMT+7", "Etc/GMT+8", "Etc/GMT+9", "Etc/GMT-1", "Etc/GMT-10", "Etc/GMT-11", "Etc/GMT-12", "Etc/GMT-2", "Etc/GMT-3", "Etc/GMT-4", "Etc/GMT-5", "Etc/GMT-6", "Etc/GMT-7", "Etc/GMT-8", "Etc/GMT-9", "Etc/UTC", "Europe/Amsterdam", "Europe/Andorra", "Europe/Astrakhan", "Europe/Athens", "Europe/Belgrade", "Europe/Berlin", "Europe/Bratislava", "Europe/Brussels", "Europe/Bucharest", "Europe/Budapest", "Europe/Busingen", "Europe/Chisinau", "Europe/Copenhagen", "Europe/Dublin", "Europe/Gibraltar", "Europe/Guernsey", "Europe/Helsinki", "Europe/Isle_of_Man", "Europe/Istanbul", "Europe/Jersey", "Europe/Kaliningrad", "Europe/Kiev", "Europe/Kirov", "Europe/Lisbon", "Europe/Ljubljana", "Europe/London", "Europe/Luxembourg", "Europe/Madrid", "Europe/Malta", "Europe/Mariehamn", "Europe/Minsk", "Europe/Monaco", "Europe/Moscow", "Europe/Oslo", "Europe/Paris", "Europe/Podgorica", "Europe/Prague", "Europe/Riga", "Europe/Rome", "Europe/Samara", "Europe/San_Marino", "Europe/Sarajevo", "Europe/Saratov", "Europe/Simferopol", "Europe/Skopje", "Europe/Sofia", "Europe/Stockholm", "Europe/Tallinn", "Europe/Tirane", "Europe/Ulyanovsk", "Europe/Uzhgorod", "Europe/Vaduz", "Europe/Vienna", "Europe/Vilnius", "Europe/Volgograd", "Europe/Warsaw", "Europe/Zagreb", "Europe/Zaporozhye", "Europe/Zurich", "Indian/Antananarivo", "Indian/Chagos", "Indian/Christmas", "Indian/Cocos", "Indian/Comoro", "Indian/Kerguelen", "Indian/Mahe", "Indian/Maldives", "Indian/Mauritius", "Indian/Mayotte", "Indian/Reunion", "Pacific/Apia", "Pacific/Auckland", "Pacific/Bougainville", "Pacific/Chatham", "Pacific/Chuuk", "Pacific/Easter", "Pacific/Efate", "Pacific/Enderbury", "Pacific/Fakaofo", "Pacific/Fiji", "Pacific/Funafuti", "Pacific/Galapagos", "Pacific/Gambier", "Pacific/Guadalcanal", "Pacific/Guam", "Pacific/Honolulu", "Pacific/Kiritimati", "Pacific/Kosrae", "Pacific/Kwajalein", "Pacific/Majuro", "Pacific/Marquesas", "Pacific/Midway", "Pacific/Nauru", "Pacific/Niue", "Pacific/Norfolk", "Pacific/Noumea", "Pacific/Pago_Pago", "Pacific/Palau", "Pacific/Pitcairn", "Pacific/Pohnpei", "Pacific/Port_Moresby", "Pacific/Rarotonga", "Pacific/Saipan", "Pacific/Tahiti", "Pacific/Tarawa", "Pacific/Tongatapu", "Pacific/Wake", "Pacific/Wallis"];
	          if (W = +W, !(-90 <= (Y = +Y) && Y <= 90 && -180 <= W && W <= 180)) throw new RangeError("invalid coordinates");
	          if (90 <= Y) return "Etc/GMT";
	          var S = -1,
	              V = 48 * (180 + W) / 360.00000000000006,
	              X = 24 * (90 - Y) / 180.00000000000003,
	              Z = 0 | V,
	              M = 0 | X,
	              I = 96 * M + 2 * Z;

	          for (I = 56 * T.charCodeAt(I) + T.charCodeAt(I + 1) - 1995; I + U.length < 3136;) {
	            I = 56 * T.charCodeAt(I = 8 * (S = S + I + 1) + 4 * (M = 0 | (X = 2 * (X - M) % 2)) + 2 * (Z = 0 | (V = 2 * (V - Z) % 2)) + 2304) + T.charCodeAt(I + 1) - 1995;
	          }

	          return U[I + U.length - 3136];
	        }

	        "undefined" != typeof module && (module.exports = tzlookup);
	      }, {}],
	      2: [function (_dereq_, module, exports) {

	        var tzlookup = _dereq_('tz-lookup'); //.trim() pollyfill


	        if (!String.prototype.trim) {
	          var rtrim = /^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g;

	          String.prototype.trim = function () {
	            return this.replace(rtrim, '');
	          };
	        }

	        var isArray = function isArray(hmm) {
	          return Object.prototype.toString.call(hmm) === '[object Array]';
	        };

	        var isString = function isString(hmm) {
	          return typeof hmm === 'string';
	        };

	        function isObject(hmm) {
	          return hmm instanceof Object && hmm.constructor === Object;
	        }

	        var findTz = function findTz(geo, b) {
	          var lat = null;
	          var lng = null; //accept weird formats

	          if (typeof b === 'number' && typeof geo === 'number') {
	            lat = geo;
	            lng = b;
	          } else if (isArray(geo) === true) {
	            lat = geo[0];
	            lng = geo[1];
	          } else if (isString(geo) === true) {
	            var arr = geo.split(/[,/]/);
	            lat = arr[0].trim();
	            lng = arr[1].trim();
	          } else if (isObject(geo) === true) {
	            lat = geo.lat || geo.latitude;
	            lng = geo.lng || geo.lon || geo["long"] || geo.longitude;
	          } else {
	            return this;
	          } //validate lat/lng


	          if (lat < -90 || lat > 90) {
	            console.warn('Invalid latitude: ' + lat);
	            return this;
	          }

	          if (lng < -180 || lng > 180) {
	            console.warn('Invalid longitude: ' + lng);
	            return this;
	          }

	          var tz = tzlookup(lat, lng);

	          if (!tz) {
	            console.warn('Found no timezone for ' + lat + ', ' + lng);
	            return this;
	          }

	          return this["goto"](tz);
	        };

	        module.exports = findTz;
	      }, {
	        "tz-lookup": 1
	      }],
	      3: [function (_dereq_, module, exports) {

	        var find = _dereq_('./findTz');

	        var point = _dereq_('./point');

	        module.exports = {
	          "in": find,
	          point: point
	        };
	      }, {
	        "./findTz": 2,
	        "./point": 5
	      }],
	      4: [function (_dereq_, module, exports) {

	        module.exports = {
	          "Pacific/Midway": '28.21,-177.37',
	          //Midway Atoll, United States
	          "Pacific/Niue": '-19.06,-169.92',
	          //Alofi, Niue
	          "Pacific/Pago_Pago": '-14.28,-170.7',
	          //Pago Pago, Eastern District, American Samoa
	          "America/Adak": '51.88,-176.66',
	          //Adak, AK, USA
	          "Pacific/Honolulu": '21.31,-157.86',
	          //Honolulu, HI, USA
	          "Pacific/Johnston": '16.73,-169.53',
	          //Johnston Island, United States Minor Outlying Islands
	          "Pacific/Rarotonga": '-21.21,-159.78',
	          //Avarua District, Cook Islands
	          "Pacific/Tahiti": '-17.56,-149.6',
	          //Fa'a'ā, French Polynesia
	          "Pacific/Marquesas": '-8.91,-140.1',
	          //Taioha'e, French Polynesia
	          "America/Anchorage": '61.22,-149.9',
	          //Anchorage, AK, USA
	          "America/Juneau": '58.3,-134.42',
	          //Juneau, AK, USA
	          "America/Metlakatla": '55.13,-131.57',
	          //Metlakatla, AK 99926, USA
	          "America/Nome": '64.5,-165.41',
	          //Nome, AK, USA
	          "America/Sitka": '57.05,-135.33',
	          //Sitka, AK, USA
	          "America/Yakutat": '59.55,-139.73',
	          //Yakutat, AK, USA
	          "Pacific/Gambier": '-23.11,-134.97',
	          //Mangareva, French Polynesia
	          "America/Dawson": '64.06,-139.43',
	          //Dawson, YT, Canada
	          "America/Los_Angeles": '34.05,-118.24',
	          //Los Angeles, CA, USA
	          "America/Tijuana": '32.51,-117.04',
	          //Tijuana, Baja California, Mexico
	          "America/Vancouver": '49.28,-123.12',
	          //Vancouver, BC, Canada
	          "America/Whitehorse": '60.72,-135.06',
	          //Whitehorse, YT, Canada
	          "Pacific/Pitcairn": '-25.07,-130.1',
	          //Adamstown PCRN 1ZZ, Pitcairn Islands
	          "America/Boise": '43.62,-116.2',
	          //Boise, ID, USA
	          "America/Cambridge_Bay": '69.12,-105.06',
	          //Cambridge Bay, NU, Canada
	          "America/Chihuahua": '28.63,-106.07',
	          //Chihuahua, Mexico
	          "America/Creston": '49.1,-116.51',
	          //Creston, BC V0B, Canada
	          "America/Dawson_Creek": '56.25,-120.85',
	          //Fort St John, BC, Canada
	          "America/Denver": '31.76,-106.49',
	          //El Paso, TX, USA
	          "America/Edmonton": '51.05,-114.07',
	          //Calgary, AB, Canada
	          "America/Fort_Nelson": '58.81,-122.7',
	          //Fort Nelson, BC V0C, Canada
	          "America/Hermosillo": '29.07,-110.96',
	          //Hermosillo, Sonora, Mexico
	          "America/Inuvik": '68.36,-133.72',
	          //Inuvik, NT X0E, Canada
	          "America/Mazatlan": '24.81,-107.39',
	          //Culiacán, Sinaloa, Mexico
	          "America/Ojinaga": '31.69,-106.42',
	          //Ciudad Juarez, Chihuahua, Mexico
	          "America/Phoenix": '33.45,-112.07',
	          //Phoenix, AZ, USA
	          "America/Yellowknife": '62.45,-114.37',
	          //Yellowknife, NT X0E, Canada
	          "America/Bahia_Banderas": '20.76,-105.33',
	          //Bucerías, Nayarit, Mexico
	          "America/Belize": '17.5,-88.2',
	          //Belize City, Belize
	          "America/Chicago": '41.88,-87.63',
	          //Chicago, IL, USA
	          "America/Costa_Rica": '9.93,-84.09',
	          //San José Province, San José, Costa Rica
	          "America/El_Salvador": '13.69,-89.22',
	          //San Salvador, El Salvador
	          "America/Guatemala": '14.63,-90.51',
	          //Guatemala City, Guatemala
	          "America/Indiana/Knox": '41.3,-86.63',
	          //Knox, IN 46534, USA
	          "America/Indiana/Tell_City": '37.95,-86.77',
	          //Tell City, IN, USA
	          "America/Managua": '12.11,-86.24',
	          //Managua, Nicaragua
	          "America/Matamoros": '26.05,-98.3',
	          //Reynosa, Tamaulipas, Mexico
	          "America/Menominee": '45.11,-87.61',
	          //Menominee, MI 49858, USA
	          "America/Merida": '20.97,-89.59',
	          //Mérida, Yucatan, Mexico
	          "America/Mexico_City": '19.43,-99.13',
	          //Mexico City, CDMX, Mexico
	          "America/Monterrey": '19.49,-99.12',
	          //Gustavo A. Madero, Mexico City, CDMX, Mexico
	          "America/North_Dakota/Beulah": '44.63,-86.09',
	          //Beulah, MI, USA
	          "America/North_Dakota/Center": '47.115,-101.301',
	          //Center, ND, USA
	          "America/North_Dakota/New_Salem": '46.83,-100.89',
	          //Mandan, ND 58554, USA
	          "America/Rainy_River": '48.72,-94.57',
	          //Rainy River, ON, Canada
	          "America/Rankin_Inlet": '62.81,-92.09',
	          //Rankin Inlet, NU, Canada
	          "America/Regina": '52.13,-106.67',
	          //Saskatoon, SK, Canada
	          "America/Resolute": '74.7,-94.83',
	          //Resolute, NU, Canada
	          "America/Swift_Current": '50.29,-107.8',
	          //Swift Current, SK, Canada
	          "America/Tegucigalpa": '14.07,-87.19',
	          //Tegucigalpa, Honduras
	          "America/Winnipeg": '49.9,-97.14',
	          //Winnipeg, MB, Canada
	          "Pacific/Easter": '-27.11,-109.35',
	          //Easter Island, Valparaiso Region, Chile
	          "Pacific/Galapagos": '-0.74,-90.32',
	          //Puerto Ayora, Ecuador
	          "America/Atikokan": '48.76,-91.62',
	          //Atikokan, ON P0W, Canada
	          "America/Bogota": '4.71,-74.07',
	          //Bogotá, Bogota, Colombia
	          "America/Cancun": '21.16,-86.85',
	          //Cancún, Quintana Roo, Mexico
	          "America/Cayman": '19.29,-81.37',
	          //George Town, Cayman Islands
	          "America/Detroit": '42.33,-83.05',
	          //Detroit, MI, USA
	          "America/Eirunepe": '-6.66,-69.87',
	          //Eirunepé - State of Amazonas, 69880-000, Brazil
	          "America/Guayaquil": '-2.17,-79.92',
	          //Guayaquil, Ecuador
	          "America/Havana": '23.11,-82.37',
	          //Havana, Cuba
	          "America/Indiana/Indianapolis": '39.77,-86.16',
	          //Indianapolis, IN, USA
	          "America/Indiana/Marengo": '42.25,-88.61',
	          //Marengo, IL 60152, USA
	          "America/Indiana/Petersburg": '37.23,-77.4',
	          //Petersburg, VA, USA
	          "America/Indiana/Vevay": '38.75,-85.07',
	          //Vevay, IN 47043, USA
	          "America/Indiana/Vincennes": '38.68,-87.53',
	          //Vincennes, IN 47591, USA
	          "America/Indiana/Winamac": '41.05,-86.6',
	          //Winamac, IN 46996, USA
	          "America/Iqaluit": '63.75,-68.52',
	          //Iqaluit, NU, Canada
	          "America/Jamaica": '18.02,-76.81',
	          //Kingston, Jamaica
	          "America/Kentucky/Louisville": '38.25,-85.76',
	          //Louisville, KY, USA
	          "America/Kentucky/Monticello": '41.66,-74.69',
	          //Monticello, NY 12701, USA
	          "America/Lima": '-12.05,-77.04',
	          //Lima, Peru
	          "America/Nassau": '25.05,-77.36',
	          //Nassau, The Bahamas
	          "America/New_York": '40.71,-74.01',
	          //New York, NY, USA
	          "America/Nipigon": '49.02,-88.27',
	          //Nipigon, ON P0T, Canada
	          "America/Panama": '8.98,-79.52',
	          //Panama City, Panama
	          "America/Pangnirtung": '66.15,-65.7',
	          //Pangnirtung, NU, Canada
	          "America/Port-au-Prince": '18.59,-72.31',
	          //Port-au-Prince, Haiti
	          "America/Rio_Branco": '-9.98,-68.43',
	          //Rio Branco - State of Acre, Brazil
	          "America/Thunder_Bay": '48.38,-89.25',
	          //Thunder Bay, ON, Canada
	          "America/Toronto": '43.65,-79.38',
	          //Toronto, ON, Canada
	          "America/Anguilla": '18.21,-63.06',
	          //The Valley 2640, Anguilla
	          "America/Antigua": '17.13,-61.85',
	          //St John's, Antigua and Barbuda
	          "America/Aruba": '12.54,-70',
	          //Babijn, Aruba
	          "America/Asuncion": '-25.26,-57.58',
	          //Asuncion, Paraguay
	          "America/Barbados": '13.1,-59.61',
	          //Bridgetown, Barbados
	          "America/Blanc-Sablon": '46.74,-71.25',
	          //Levis, QC, Canada
	          "America/Boa_Vista": '2.82,-60.68',
	          //Boa Vista, State of Roraima, Brazil
	          "America/Campo_Grande": '-20.47,-54.62',
	          //Campo Grande, State of Mato Grosso do Sul, Brazil
	          "America/Caracas": '10.48,-66.9',
	          //Caracas, Capital District, Venezuela
	          "America/Cuiaba": '-15.6,-56.1',
	          //Cuiabá - Coxipó da Ponte, Cuiabá - State of Mato Grosso, Brazil
	          "America/Curacao": '12.12,-68.88',
	          //Willemstad, Curaçao
	          "America/Dominica": '15.31,-61.38',
	          //Roseau, Dominica
	          "America/Glace_Bay": '46.14,-60.19',
	          //Sydney, NS, Canada
	          "America/Goose_Bay": '52.94,-66.91',
	          //Labrador City, NL, Canada
	          "America/Grand_Turk": '21.47,-71.14',
	          //Cockburn Town TKCA 1ZZ, Turks and Caicos Islands
	          "America/Grenada": '12.06,-61.75',
	          //St George's, Grenada
	          "America/Guadeloupe": '16.27,-61.53',
	          //Les Abymes, Guadeloupe
	          "America/Guyana": '6.8,-58.16',
	          //Georgetown, Guyana
	          "America/Halifax": '44.65,-63.58',
	          //Halifax Regional Municipality, NS, Canada
	          "America/Kralendijk": '12.14,-68.27',
	          //Kralendijk, Caribbean Netherlands
	          "America/La_Paz": '-17.81,-63.16',
	          //Santa Cruz de la Sierra, Bolivia
	          "America/Lower_Princes": '18.05,-63.06',
	          //Dutch Cul de Sac, Sint Maarten
	          "America/Manaus": '-3.12,-60.02',
	          //Manaus, State of Amazonas, Brazil
	          "America/Marigot": '18.07,-63.08',
	          //Marigot, St Martin
	          "America/Martinique": '14.62,-61.06',
	          //Fort-de-France Bay, Martinique
	          "America/Moncton": '45.27,-66.06',
	          //Saint John, NB, Canada
	          "America/Montserrat": '16.79,-62.21',
	          //Brades, Montserrat
	          "America/Porto_Velho": '-8.76,-63.9',
	          //Porto Velho - State of Rondônia, Brazil
	          "America/Port_of_Spain": '10.65,-61.5',
	          //Laventille, Port of Spain, Trinidad and Tobago
	          "America/Puerto_Rico": '18.47,-66.11',
	          //San Juan, Puerto Rico
	          "America/Santiago": '-33.45,-70.67',
	          //Santiago, Santiago Metropolitan Region, Chile
	          "America/Santo_Domingo": '18.49,-69.93',
	          //Santo Domingo, Dominican Republic
	          "America/St_Barthelemy": '17.9,-62.85',
	          //Gustavia, St Barthélemy
	          "America/St_Kitts": '17.3,-62.72',
	          //Basseterre, St Kitts & Nevis
	          "America/St_Lucia": '14.01,-60.99',
	          //Castries, St Lucia
	          "America/St_Thomas": '17.72,-64.83',
	          //St Croix, USVI
	          "America/St_Vincent": '13.16,-61.22',
	          //Kingstown, St Vincent and the Grenadines
	          "America/Thule": '77.47,-69.23',
	          //Qaanaaq, Greenland
	          "America/Tortola": '18.43,-64.63',
	          //Tortola, British Virgin Islands
	          "Antarctica/Palmer": '-64.77,-64.05',
	          //Antarctica
	          "Atlantic/Bermuda": '32.29,-64.79',
	          //Hamilton, Bermuda
	          "America/St_Johns": '47.56,-52.71',
	          //St. John's, NL, Canada
	          "America/Araguaina": '-10.25,-48.32',
	          //Palmas, TO, Brazil
	          "America/Argentina/Buenos_Aires": '-34.6,-58.38',
	          //Buenos Aires, Argentina
	          "America/Argentina/Catamarca": '-28.47,-65.78',
	          //Catamarca, Catamarca Province, Argentina
	          "America/Argentina/Cordoba": '-31.42,-64.19',
	          //Córdoba, Cordoba, Argentina
	          "America/Argentina/Jujuy": '-24.19,-65.3',
	          //Jujuy, Argentina
	          "America/Argentina/La_Rioja": '-29.41,-66.86',
	          //La Rioja, La Rioja Province, Argentina
	          "America/Argentina/Mendoza": '-32.89,-68.85',
	          //Mendoza, Capital Department, Mendoza Province, Argentina
	          "America/Argentina/Rio_Gallegos": '-51.62,-69.22',
	          //Rio Gallegos, Santa Cruz Province, Argentina
	          "America/Argentina/Salta": '-24.78,-65.42',
	          //Salta, Salta Province, Argentina
	          "America/Argentina/San_Juan": '-31.54,-68.54',
	          //San Juan, San Juan Province, Argentina
	          "America/Argentina/San_Luis": '-33.3,-66.34',
	          //San Luis, San Luis Province, Argentina
	          "America/Argentina/Tucuman": '-26.81,-65.22',
	          //San Miguel de Tucumán, Tucumán, Argentina
	          "America/Argentina/Ushuaia": '-54.8,-68.3',
	          //Ushuaia, Tierra del Fuego Province, Argentina
	          "America/Bahia": '-12.98,-38.5',
	          //Salvador - State of Bahia, Brazil
	          "America/Belem": '-1.46,-48.49',
	          //Belém - State of Pará, Brazil
	          "America/Cayenne": '4.92,-52.31',
	          //Cayenne, French Guiana
	          "America/Fortaleza": '-3.73,-38.53',
	          //Fortaleza - State of Ceará, Brazil
	          "America/Godthab": '64.18,-51.69',
	          //Nuuk, Greenland
	          "America/Maceio": '-9.65,-35.71',
	          //Maceió - Jardim da Saúde, Maceió - State of Alagoas, Brazil
	          "America/Miquelon": '46.78,-56.18',
	          //Saint-Pierre, St Pierre and Miquelon
	          "America/Montevideo": '-34.9,-56.16',
	          //Montevideo, Montevideo Department, Uruguay
	          "America/Paramaribo": '5.85,-55.2',
	          //Paramaribo, Suriname
	          "America/Recife": '-8.05,-34.93',
	          //Recife - State of Pernambuco, Brazil
	          "America/Santarem": '-2.45,-54.7',
	          //Santarém, PA, Brazil
	          "America/Sao_Paulo": '-23.55,-46.63',
	          //São Paulo, State of São Paulo, Brazil
	          "Antarctica/Rothera": '-67.56,-68.12',
	          //BIQQ 1ZZ, Antarctica
	          "Atlantic/Stanley": '-51.7,-57.85',
	          //Stanley FIQQ 1ZZ, Falkland Islands (Islas Malvinas)
	          "America/Noronha": '-7.75,-34.83',
	          //Ilha de Itamaracá - Baixa Verde, Itamaracá - State of Pernambuco, Brazil
	          "Atlantic/South_Georgia": '-54.28,-36.51',
	          //Grytviken SIQQ 1ZZ, South Georgia and the South Sandwich Islands
	          "America/Scoresbysund": '70.5,-25',
	          //Scoresby Sund, Greenland
	          "Atlantic/Azores": '37.74,-25.68',
	          //Ponta Delgada, Portugal
	          "Atlantic/Cape_Verde": '14.93,-23.51',
	          //Praia, Cape Verde
	          "Africa/Abidjan": '5.36,-4.01',
	          //Abidjan, Côte d'Ivoire
	          "Africa/Accra": '5.6,-0.19',
	          //Accra, Ghana
	          "Africa/Bamako": '12.64,-8',
	          //Bamako, Mali
	          "Africa/Banjul": '13.27,-16.64',
	          //Brikama, The Gambia
	          "Africa/Bissau": '11.88,-15.62',
	          //Bissau, Guinea-Bissau
	          "Africa/Casablanca": '33.57,-7.59',
	          //Casablanca, Morocco
	          "Africa/Conakry": '9.53,-13.69',
	          //Camayenne, Conakry, Guinea
	          "Africa/Dakar": '14.72,-17.47',
	          //Dakar, Senegal
	          "Africa/El_Aaiun": '23.72,-15.93',
	          //Dakhla 73000
	          "Africa/Freetown": '8.47,-13.23',
	          //Freetown, Sierra Leone
	          "Africa/Lome": '6.17,1.23',
	          //Lome, Togo
	          "Africa/Monrovia": '6.29,-10.76',
	          //Monrovia, Liberia
	          "Africa/Nouakchott": '18.07,-15.96',
	          //Nouakchott, Mauritania
	          "Africa/Ouagadougou": '12.37,-1.52',
	          //Ouagadougou, Burkina Faso
	          "America/Danmarkshavn": '74.48,-18.98',
	          //Greenland
	          "Antarctica/Troll": '-72.01,2.54',
	          //Troll, Antarctica
	          "Atlantic/Canary": '28.12,-15.44',
	          //Las Palmas de Gran Canaria, Las Palmas, Spain
	          "Atlantic/Faroe": '62.01,-6.79',
	          //Tórshavn, Faroe Islands
	          "Atlantic/Madeira": '32.67,-16.92',
	          //Funchal, Portugal
	          "Atlantic/Reykjavik": '64.13,-21.82',
	          //Reykjavík, Iceland
	          "Atlantic/St_Helena": '-15.93,-5.72',
	          //Jamestown STHL 1ZZ, St Helena, Ascension and Tristan da Cunha
	          "Europe/Dublin": '53.35,-6.26',
	          //Dublin, Ireland
	          "Europe/Guernsey": '49.45,-2.55',
	          //Saint Peter Port, Guernsey
	          "Europe/Isle_of_Man": '54.15,-4.49',
	          //Douglas, Isle of Man
	          "Europe/Jersey": '49.18,-2.1',
	          //St Helier, Jersey
	          "Europe/Lisbon": '38.72,-9.14',
	          //Lisbon, Portugal
	          "Europe/London": '51.51,-0.13',
	          //London, UK
	          "Africa/Algiers": '36.75,3.06',
	          //Algiers, Sidi M'Hamed, Algeria
	          "Africa/Bangui": '4.39,18.56',
	          //Bangui, Central African Republic
	          "Africa/Brazzaville": '-4.26,15.24',
	          //Brazzaville, Republic of the Congo
	          "Africa/Ceuta": '35.89,-5.32',
	          //Ceuta, Spain
	          "Africa/Douala": '4.05,9.77',
	          //Douala, Cameroon
	          "Africa/Kinshasa": '-4.44,15.27',
	          //Kinshasa, Democratic Republic of the Congo
	          "Africa/Lagos": '6.52,3.38',
	          //Lagos, Nigeria
	          "Africa/Libreville": '0.42,9.47',
	          //Libreville, Gabon
	          "Africa/Luanda": '-8.84,13.29',
	          //Luanda, Angola
	          "Africa/Malabo": '1.85,9.78',
	          //Bata, Equatorial Guinea
	          "Africa/Ndjamena": '12.13,15.06',
	          //N'Djamena, Chad
	          "Africa/Niamey": '13.51,2.13',
	          //Niamey, Niger
	          "Africa/Porto-Novo": '6.37,2.39',
	          //Cotonou, Benin
	          "Africa/Sao_Tome": '0.33,6.73',
	          //São Tomé, São Tomé and Príncipe
	          "Africa/Tunis": '36.81,10.18',
	          //Tunis, Tunisia
	          "Africa/Windhoek": '-22.56,17.07',
	          //Windhoek, Namibia
	          "Arctic/Longyearbyen": '78.22,15.63',
	          //Longyearbyen, Svalbard and Jan Mayen
	          "Europe/Amsterdam": '52.37,4.9',
	          //Amsterdam, Netherlands
	          "Europe/Andorra": '42.51,1.52',
	          //AD500 Andorra la Vella, Andorra
	          "Europe/Belgrade": '44.79,20.45',
	          //Belgrade, Serbia
	          "Europe/Berlin": '52.52,13.4',
	          //Berlin, Germany
	          "Europe/Bratislava": '48.15,17.11',
	          //Bratislava, Slovakia
	          "Europe/Brussels": '50.85,4.35',
	          //Brussels, Belgium
	          "Europe/Budapest": '47.5,19.04',
	          //Budapest, Hungary
	          "Europe/Copenhagen": '55.68,12.57',
	          //Copenhagen, Denmark
	          "Europe/Gibraltar": '36.14,-5.35',
	          //Gibraltar
	          "Europe/Ljubljana": '46.06,14.51',
	          //Ljubljana, Slovenia
	          "Europe/Luxembourg": '49.61,6.13',
	          //Luxembourg City, Luxembourg
	          "Europe/Madrid": '40.42,-3.7',
	          //Madrid, Spain
	          "Europe/Malta": '35.9,14.47',
	          //Birkirkara, Malta
	          "Europe/Monaco": '43.74,7.42',
	          //Monaco
	          "Europe/Oslo": '59.91,10.75',
	          //Oslo, Norway
	          "Europe/Paris": '48.86,2.35',
	          //Paris, France
	          "Europe/Podgorica": '42.43,19.26',
	          //Podgorica, Montenegro
	          "Europe/Prague": '50.08,14.44',
	          //Prague, Czechia
	          "Europe/Rome": '41.9,12.5',
	          //Rome, Metropolitan City of Rome, Italy
	          "Europe/San_Marino": '43.97,12.48',
	          //Serravalle, San Marino
	          "Europe/Sarajevo": '43.86,18.41',
	          //Sarajevo, Bosnia and Herzegovina
	          "Europe/Skopje": '42,21.43',
	          //Skopje, Macedonia (FYROM)
	          "Europe/Stockholm": '59.33,18.07',
	          //Stockholm, Sweden
	          "Europe/Tirane": '41.33,19.82',
	          //Tirana, Albania
	          "Europe/Vaduz": '47.17,9.51',
	          //Schaan, Liechtenstein
	          "Europe/Vatican": '41.9,12.45',
	          //00120, Vatican City
	          "Europe/Vienna": '48.21,16.37',
	          //Vienna, Austria
	          "Europe/Warsaw": '52.23,21.01',
	          //Warsaw, Poland
	          "Europe/Zagreb": '45.81,15.97',
	          //u. l., Gundulićeva ul. 43, 10000, Zagreb, Croatia
	          "Europe/Zurich": '47.38,8.54',
	          //Zürich, Switzerland
	          "Africa/Blantyre": '-13.96,33.77',
	          //Lilongwe, Malawi
	          "Africa/Bujumbura": '-3.36,29.36',
	          //Bujumbura, Burundi
	          "Africa/Cairo": '30.04,31.24',
	          //Cairo, Cairo Governorate, Egypt
	          "Africa/Gaborone": '-24.63,25.92',
	          //Gaborone, Botswana
	          "Africa/Harare": '-17.83,31.03',
	          //Harare, Zimbabwe
	          "Africa/Johannesburg": '-33.92,18.42',
	          //Cape Town, South Africa
	          "Africa/Khartoum": '15.5,32.56',
	          //Khartoum, Sudan
	          "Africa/Kigali": '-1.97,30.1',
	          //Kigali, Rwanda
	          "Africa/Lubumbashi": '-11.69,27.5',
	          //Lubumbashi, Democratic Republic of the Congo
	          "Africa/Lusaka": '-15.39,28.32',
	          //Lusaka, Zambia
	          "Africa/Maputo": '-25.97,32.57',
	          //Maputo, Mozambique
	          "Africa/Maseru": '-29.32,27.49',
	          //Maseru, Lesotho
	          "Africa/Mbabane": '-26.51,31.37',
	          //Manzini, Swaziland
	          "Africa/Tripoli": '32.89,13.19',
	          //Tripoli, Libya
	          "Asia/Amman": '31.95,35.93',
	          //Amman, Jordan
	          "Asia/Beirut": '33.89,35.5',
	          //Beirut, Lebanon
	          "Asia/Damascus": '36.2,37.13',
	          //Aleppo, Syria
	          "Asia/Gaza": '31.35,34.31',
	          //Gaza Strip
	          "Asia/Hebron": '31.77,35.21',
	          //Jerusalem, Israel
	          "Asia/Jerusalem": '31.77,35.21',
	          //Jerusalem, Israel
	          "Asia/Nicosia": '35.19,33.38',
	          //Nicosia, Cyprus
	          "Europe/Athens": '37.98,23.73',
	          //Athens, Greece
	          "Europe/Bucharest": '44.43,26.1',
	          //Bucharest, Romania
	          "Europe/Chisinau": '47.01,28.86',
	          //Chisinau, Moldova
	          "Europe/Helsinki": '60.17,24.94',
	          //Helsinki, Finland
	          "Europe/Istanbul": '41.01,28.98',
	          //Istanbul, Turkey
	          "Europe/Kaliningrad": '54.71,20.45',
	          //Kaliningrad, Kaliningrad Oblast, Russia
	          "Europe/Kiev": '50.45,30.52',
	          //Kyiv, Ukraine, 02000
	          "Europe/Mariehamn": '60.1,19.93',
	          //Mariehamn, Åland Islands
	          "Europe/Riga": '56.95,24.11',
	          //Riga, Latvia
	          "Europe/Sofia": '42.7,23.32',
	          //Sofia, Bulgaria
	          "Europe/Tallinn": '59.44,24.75',
	          //Tallinn, Estonia
	          "Europe/Uzhgorod": '48.62,22.29',
	          //Uzhhorod, Zakarpats'ka oblast, Ukraine
	          "Europe/Vilnius": '54.69,25.28',
	          //Vilnius, Lithuania
	          "Europe/Zaporozhye": '47.84,35.14',
	          //Zaporizhzhia, Zaporiz'ka oblast, Ukraine, 69061
	          "Africa/Addis_Ababa": '8.98,38.76',
	          //Addis Ababa, Ethiopia
	          "Africa/Asmara": '15.32,38.93',
	          //Asmara, Eritrea
	          "Africa/Dar_es_Salaam": '-6.79,39.21',
	          //Dar es Salaam, Tanzania
	          "Africa/Djibouti": '11.83,42.59',
	          //Djibouti
	          "Africa/Juba": '4.86,31.57',
	          //Juba, South Sudan
	          "Africa/Kampala": '0.35,32.58',
	          //Kampala, Uganda
	          "Africa/Mogadishu": '2.05,45.32',
	          //Mogadishu, Somalia
	          "Africa/Nairobi": '-1.29,36.82',
	          //Nairobi, Kenya
	          "Antarctica/Syowa": '-69,39.58',
	          //Antarctica
	          "Asia/Aden": '15.37,44.19',
	          //Sana'a, Yemen
	          "Asia/Baghdad": '33.31,44.36',
	          //Baghdad, Iraq
	          "Asia/Bahrain": '26.23,50.59',
	          //Manama, Bahrain
	          "Asia/Kuwait": '29.09,48.07',
	          //Ahmadi, Kuwait
	          "Asia/Qatar": '25.29,51.53',
	          //Doha, Qatar
	          "Asia/Riyadh": '24.71,46.68',
	          //Riyadh Saudi Arabia
	          "Europe/Minsk": '53.9,27.56',
	          //Minsk, Belarus
	          "Europe/Moscow": '55.76,37.62',
	          //Moscow, Russia
	          "Europe/Simferopol": '44.62,33.53',
	          //Sevastopol 99000
	          "Europe/Volgograd": '48.71,44.51',
	          //Volgograd, Volgograd Oblast, Russia
	          "Indian/Antananarivo": '-18.88,47.51',
	          //Antananarivo, Madagascar
	          "Indian/Comoro": '-11.72,43.25',
	          //Moroni, Comoros
	          "Indian/Mayotte": '-12.78,45.23',
	          //Mamoudzou, Mayotte
	          "Asia/Tehran": '35.69,51.39',
	          //Tehran, Tehran Province, Iran
	          "Asia/Baku": '40.41,49.87',
	          //Baku, Azerbaijan
	          "Asia/Dubai": '25.2,55.27',
	          //Dubai - United Arab Emirates
	          "Asia/Muscat": '23.59,58.41',
	          //Muscat, Oman
	          "Asia/Tbilisi": '41.72,44.83',
	          //Tbilisi, Georgia
	          "Asia/Yerevan": '40.18,44.5',
	          //Yerevan, Armenia
	          "Europe/Astrakhan": '46.36,48.06',
	          //Astrakhan, Astrakhan Oblast, Russia
	          "Europe/Samara": '53.24,50.22',
	          //Samara, Samara Oblast, Russia
	          "Europe/Ulyanovsk": '54.32,48.38',
	          //Ulyanovsk, Ulyanovsk Oblast, Russia
	          "Indian/Mahe": '-4.62,55.45',
	          //Victoria, Seychelles
	          "Indian/Mauritius": '-20.16,57.5',
	          //Port Louis, Mauritius
	          "Indian/Reunion": '-20.89,55.46',
	          //Saint-Denis, Reunion
	          "Asia/Kabul": '34.56,69.21',
	          //Kabul, Afghanistan
	          "Antarctica/Mawson": '-67.6,62.87',
	          //Antarctica
	          "Asia/Aqtau": '43.64,51.2',
	          //Aktau, Kazakhstan
	          "Asia/Aqtobe": '50.28,57.17',
	          //Aktobe, Kazakhstan
	          "Asia/Ashgabat": '37.96,58.33',
	          //Ashgabat, Turkmenistan
	          "Asia/Dushanbe": '38.56,68.79',
	          //Dushanbe, Tajikistan
	          "Asia/Karachi": '24.86,67',
	          //Karachi, Karachi City, Sindh, Pakistan
	          "Asia/Oral": '51.23,51.39',
	          //Uralsk, Kazakhstan
	          "Asia/Samarkand": '39.63,66.97',
	          //Samarkand, Uzbekistan
	          "Asia/Tashkent": '41.3,69.24',
	          //Tashkent, Uzbekistan
	          "Asia/Yekaterinburg": '56.84,60.61',
	          //Yekaterinburg, Sverdlovsk Oblast, Russia
	          "Indian/Kerguelen": '-49.35,70.22',
	          //Port-aux-Français, French Southern and Antarctic Lands
	          "Indian/Maldives": '4.18,73.51',
	          //Malé, Maldives
	          "Asia/Colombo": '6.93,79.86',
	          //Colombo, Sri Lanka
	          "Asia/Kolkata": '19.08,72.88',
	          //Mumbai, Maharashtra, India
	          "Asia/Kathmandu": '27.72,85.32',
	          //Kathmandu 44600, Nepal
	          "Antarctica/Vostok": '-78.46,106.83',
	          //Antarctica
	          "Asia/Almaty": '43.22,76.85',
	          //Almaty, Kazakhstan
	          "Asia/Bishkek": '42.87,74.57',
	          //Bishkek, Kyrgyzstan
	          "Asia/Dhaka": '23.81,90.41',
	          //Dhaka, Bangladesh
	          "Asia/Novosibirsk": '55.01,82.94',
	          //Novosibirsk, Novosibirsk Oblast, Russia
	          "Asia/Omsk": '54.99,73.32',
	          //Omsk, Omsk Oblast, Russia
	          "Asia/Qyzylorda": '44.85,65.48',
	          //Kyzylorda, Kazakhstan
	          "Asia/Thimphu": '27.47,89.64',
	          //Thimphu, Bhutan
	          "Asia/Urumqi": '22.52,113.39',
	          //Zhongshan, Guangdong, China
	          "Indian/Chagos": '-6.34,71.86',
	          //Chagos Archipelago, British Indian Ocean Territory
	          "Asia/Rangoon": '16.87,96.2',
	          //Yangon, Myanmar (Burma)
	          "Indian/Cocos": '-12.15,96.82',
	          //West Island, 6799, Cocos (Keeling) Islands
	          "Antarctica/Davis": '-68.58,77.97',
	          //Davis Station, Antarctica
	          "Asia/Bangkok": '13.76,100.5',
	          //Bangkok, Thailand
	          "Asia/Barnaul": '53.35,83.77',
	          //Barnaul, Altai Krai, Russia
	          "Asia/Hovd": '47.98,91.63',
	          //Khovd, Mongolia
	          "Asia/Ho_Chi_Minh": '10.82,106.63',
	          //Ho Chi Minh City, Vietnam
	          "Asia/Jakarta": '-6.18,106.87',
	          //Jakarta, Indonesia
	          "Asia/Krasnoyarsk": '56.02,92.89',
	          //Krasnoyarsk, Krasnoyarsk Krai, Russia
	          "Asia/Novokuznetsk": '53.76,87.12',
	          //Novokuznetsk, Kemerovo Oblast, Russia
	          "Asia/Phnom_Penh": '11.56,104.93',
	          //Phnom Penh, Cambodia
	          "Asia/Pontianak": '-0.03,109.34',
	          //Pontianak, West Kalimantan, Indonesia
	          "Asia/Vientiane": '17.98,102.63',
	          //Vientiane, Laos
	          "Indian/Christmas": '-10.43,105.67',
	          //Flying Fish Cove, Christmas Island
	          "Antarctica/Casey": '-66.28,110.53',
	          //Casey Station, Antarctica
	          "Asia/Brunei": '4.9,114.94',
	          //Bandar Seri Begawan, Brunei
	          "Asia/Choibalsan": '46.67,113.29',
	          //Baruun-Urt, Mongolia
	          "Asia/Hong_Kong": '22.4,114.11',
	          //Hong Kong
	          "Asia/Irkutsk": '52.29,104.31',
	          //Irkutsk, Irkutsk Oblast, Russia
	          "Asia/Kuala_Lumpur": '6.12,102.25',
	          //15200 Kota Bharu, Kelantan, Malaysia
	          "Asia/Kuching": '1.55,110.36',
	          //Kuching, Sarawak, Malaysia
	          "Asia/Macau": '22.2,113.54',
	          //Macau
	          "Asia/Makassar": '-5.15,119.43',
	          //Makassar, Makassar City, South Sulawesi, Indonesia
	          "Asia/Manila": '14.68,121.04',
	          //Quezon City, Metro Manila, Philippines
	          "Asia/Shanghai": '31.23,121.47',
	          //Shanghai, China
	          "Asia/Singapore": '1.35,103.82',
	          //Singapore
	          "Asia/Taipei": '25.03,121.57',
	          //Taipei, Taiwan
	          "Asia/Ulaanbaatar": '47.89,106.91',
	          //Ulaanbaatar, Mongolia
	          "Australia/Perth": '-31.95,115.86',
	          //Perth WA, Australia
	          "Asia/Pyongyang": '39.04,125.76',
	          //Pyongyang, North Korea
	          "Australia/Eucla": '-31.68,128.89',
	          //Eucla WA 6443, Australia
	          "Asia/Chita": '52.05,113.47',
	          //Chita, Zabaykalsky Krai, Russia
	          "Asia/Dili": '-8.56,125.56',
	          //Díli, Timor-Leste
	          "Asia/Jayapura": '-3.63,128.11',
	          //Ambon Island, Maluku, Indonesia
	          "Asia/Khandyga": '62.66,135.55',
	          //Khandyga, Sakha Republic, Russia, 678720
	          "Asia/Seoul": '37.57,126.98',
	          //Seoul, South Korea
	          "Asia/Tokyo": '35.69,139.69',
	          //Tokyo, Japan
	          "Asia/Yakutsk": '52.05,113.47',
	          //Chita, Zabaykalsky Krai, Russia
	          "Pacific/Palau": '7.34,134.49',
	          //Koror, Palau
	          "Australia/Adelaide": '-34.93,138.6',
	          //Adelaide SA, Australia
	          "Australia/Broken_Hill": '-31.95,141.45',
	          //Broken Hill NSW 2880, Australia
	          "Australia/Darwin": '-12.46,130.85',
	          //Darwin NT, Australia
	          "Antarctica/DumontDUrville": '-66.66,140',
	          //Dumont d'Urville Station, Antarctica
	          "Asia/Ust-Nera": '64.56,143.22',
	          //Ust-Nera, Sakha Republic, Russia, 678730
	          "Asia/Vladivostok": '43.12,131.89',
	          //Vladivostok, Primorsky Krai, Russia
	          "Australia/Brisbane": '-27.47,153.03',
	          //Brisbane QLD, Australia
	          "Australia/Currie": '-39.93,143.85',
	          //Currie TAS 7256, Australia
	          "Australia/Hobart": '-42.88,147.33',
	          //Hobart TAS 7000, Australia
	          "Australia/Lindeman": '-20.44,149.04',
	          //Lindeman Island, Queensland 4805, Australia
	          "Australia/Melbourne": '-37.81,144.96',
	          //Melbourne VIC, Australia
	          "Australia/Sydney": '-33.87,151.21',
	          //Sydney NSW, Australia
	          "Pacific/Chuuk": '7.44,151.86',
	          //Weno, Chuuk, Federated States of Micronesia
	          "Pacific/Guam": '13.55,144.85',
	          //Dededo, Guam
	          "Pacific/Port_Moresby": '-9.44,147.18',
	          //Port Moresby, Papua New Guinea
	          "Pacific/Saipan": '15.19,145.75',
	          //Saipan, CNMI
	          "Australia/Lord_Howe": '-31.56,159.08',
	          //Lord Howe Island, New South Wales, Australia
	          "Antarctica/Macquarie": '-54.63,158.86',
	          //Macquarie
	          "Asia/Magadan": '59.56,150.83',
	          //Magadan, Magadan Oblast, Russia
	          "Asia/Sakhalin": '46.96,142.73',
	          //Yuzhno-Sakhalinsk, Sakhalin Oblast, Russia
	          "Asia/Srednekolymsk": '67.44,153.73',
	          //Srednekolymsk, Sakha Republic, Russia, 678790
	          "Pacific/Bougainville": '-6.23,155.57',
	          //Arawa, Papua New Guinea
	          "Pacific/Efate": '-17.73,168.33',
	          //Port Vila, Vanuatu
	          "Pacific/Guadalcanal": '-9.45,159.97',
	          //Honiara, Solomon Islands
	          "Pacific/Kosrae": '5.33,163.01',
	          //Tofol, Kosrae, Federated States of Micronesia
	          "Pacific/Norfolk": '-29.06,167.96',
	          //Kingston, Norfolk Island
	          "Pacific/Noumea": '-22.27,166.44',
	          //Noumea, New Caledonia
	          "Pacific/Pohnpei": '6.96,158.21',
	          //Kolonia, Pohnpei, Federated States of Micronesia
	          "Antarctica/McMurdo": '-77.84,166.69',
	          //McMurdo Station, Antarctica
	          "Asia/Anadyr": '64.73,177.5',
	          //Anadyr, Chukotka Autonomous Okrug, Russia, 689000
	          "Asia/Kamchatka": '53.04,158.68',
	          //Petropavlovsk-Kamchatskiy, Kamchatka Krai, Russia
	          "Pacific/Auckland": '-36.85,174.76',
	          //Auckland, New Zealand
	          "Pacific/Fiji": '-18.12,178.45',
	          //Suva, Fiji
	          "Pacific/Funafuti": '-8.52,179.2',
	          //Funafuti, Tuvalu
	          "Pacific/Kwajalein": '8.78,167.74',
	          //Ebeye, Kwajalein Atoll, RMI
	          "Pacific/Majuro": '7.12,171.19',
	          //Majuro, Majuro Atoll, RMI
	          "Pacific/Nauru": '-0.55,166.92',
	          //Yaren, Nauru
	          "Pacific/Tarawa": '1.45,172.97',
	          //Tarawa, Kiribati
	          "Pacific/Wake": '19.28,166.65',
	          //Wake Island
	          "Pacific/Wallis": '-13.28,-176.18',
	          //Matā'utu, Wallis and Futuna
	          "Pacific/Chatham": '-35.27,174.08',
	          //Waitangi 0293, New Zealand
	          "Pacific/Apia": '-13.85,-171.75',
	          //Apia, Samoa
	          "Pacific/Enderbury": '-3.13,-171.08',
	          //Enderbury Island, Kiribati
	          "Pacific/Fakaofo": '-8.53,-172.52',
	          //Atafu Village, Tokelau
	          "Pacific/Tongatapu": '-21.14,-175.2',
	          //Nuku'alofa, Tonga
	          "Pacific/Kiritimati": '1.87,-157.43',
	          //Kiritimati, Kiribati
	          //extra ones
	          "Africa/Asmera": '15.32,38.93',
	          //Asmara, Eritrea
	          "Africa/Timbuktu": '16.77,-3',
	          //Timbuktu, Mali
	          "America/Argentina": '-38.42,-63.62',
	          //Argentina
	          "America/Atka": '52.2,-174.2',
	          //Atka, AK, USA
	          "America/Buenos_Aires": '-34.6,-58.38',
	          //Buenos Aires, Argentina
	          "America/Catamarca": '-28.47,-65.79',
	          //Catamarca Province, Argentina
	          "America/Coral_Harbour": '64.14,-83.17',
	          //Coral Harbour, NU, Canada
	          "America/Cordoba": '-31.42,-64.18',
	          //Córdoba, Argentina
	          "America/Ensenada": '31.87,-116.6',
	          //Ensenada, Baja California, Mexico
	          "America/Fort_Wayne": '41.08,-85.14',
	          //Fort Wayne, IN, USA
	          "America/Indiana": '40.27,-86.13',
	          //Indiana, USA
	          "America/Indianapolis": '39.77,-86.16',
	          //Indianapolis, IN, USA
	          "America/Jujuy": '-24.18,-65.3',
	          //Jujuy, Argentina
	          "America/Kentucky": '37.84,-84.27',
	          //Kentucky, USA
	          "America/Knox_IN": '37.92,-85.96',
	          //Fort Knox, KY 40121, USA
	          "America/Louisville": '38.25,-85.76',
	          //Louisville, KY, USA
	          "America/Mendoza": '-32.89,-68.85',
	          //Mendoza, Capital Department, Mendoza Province, Argentina
	          "America/Montreal": '45.5,-73.57',
	          //Montreal, QC, Canada
	          "America/North_Dakota": '47.55,-101',
	          //North Dakota, USA
	          "America/Porto_Acre": '-9.59,-67.54',
	          //Porto Acre - State of Acre, 69921-000, Brazil
	          "America/Rosario": '48.64,-122.87',
	          //1400 Rosario Rd, Eastsound, WA 98245, USA
	          "America/Santa_Isabel": '32.63,-115.58',
	          //Santa Isabel, Baja California, Mexico
	          "America/Shiprock": '36.79,-108.69',
	          //Shiprock, NM, USA
	          "America/Virgin": '10.6573,-61.5180',
	          //Port of Spain
	          "Antarctica/South_Pole": '-72.29,0.7',
	          //Antarctica
	          "Asia/Ashkhabad": '37.96,58.33',
	          //Ashgabat, Turkmenistan
	          "Asia/Calcutta": '22.57,88.36',
	          //Kolkata, West Bengal, India
	          "Asia/Chongqing": '29.43,106.91',
	          //Chongqing, China
	          "Asia/Chungking": '29.43,106.91',
	          //Chongqing, China
	          "Asia/Dacca": '23.81,90.41',
	          //Dhaka, Bangladesh
	          "Asia/Harbin": '45.8,126.53',
	          //Harbin, Heilongjiang, China
	          "Asia/Istanbul": '41.01,28.98',
	          //Istanbul, Turkey
	          "Asia/Kashgar": '39.47,75.99',
	          //Kaxgar, Kashgar, Xinjiang, China
	          "Asia/Katmandu": '27.72,85.32',
	          //Kathmandu 44600, Nepal
	          "Asia/Macao": '22.2,113.54',
	          //Macau
	          "Asia/Saigon": '10.82,106.63',
	          //Ho Chi Minh City, Vietnam
	          "Asia/Tel_Aviv": '32.09,34.78',
	          //Tel Aviv-Yafo, Israel
	          "Asia/Thimbu": '27.47,89.64',
	          //Thimphu, Bhutan
	          "Asia/Ujung_Pandang": '-5.15,119.43',
	          //Makassar, Makassar City, South Sulawesi, Indonesia
	          "Asia/Ulan_Bator": '47.89,106.91',
	          //Ulaanbaatar, Mongolia
	          "Atlantic/Faeroe": '62,-7',
	          //Faroe Islands
	          "Atlantic/Jan_Mayen": '71.03,-8.29',
	          //Jan Mayen, Svalbard and Jan Mayen
	          "Australia/ACT": '-35.28,149.13',
	          //Canberra ACT 2601, Australia
	          "Australia/Canberra": '-35.28,149.13',
	          //Canberra ACT 2601, Australia
	          "Australia/LHI": '-31.56,159.08',
	          //Lord Howe Island, New South Wales, Australia
	          "Australia/NSW": '-31.25,146.92',
	          //New South Wales, Australia
	          "Australia/North": '-12.46,130.85',
	          //Darwin NT, Australia
	          "Australia/Queensland": '-20.92,142.7',
	          //Queensland, Australia
	          "Australia/South": '-30,136.21',
	          //South Australia, Australia
	          "Australia/Tasmania": '-41.45,145.97',
	          //Tasmania, Australia
	          "Australia/Victoria": '-37.47,144.79',
	          //Victoria, Australia
	          "Australia/West": '-31.95,115.86',
	          //Perth WA, Australia
	          "Australia/Yancowinna": '-31.7,141.6',
	          //Yancowinna, Australia
	          "Brazil/Acre": '-9.02,-70.81',
	          //State of Acre, Brazil
	          "Brazil/DeNoronha": '-3.84,-32.43',
	          //Fernando de Noronha, State of Pernambuco, 53990-000, Brazil
	          "Brazil/East": '-22.662,-48.635',
	          //Sao Paulo
	          "Brazil/West": '-2.573,-59.981',
	          //Central-West Region, Brazil
	          "Canada/Atlantic": '44.65,-63.58',
	          //Halifax Regional Municipality, NS, Canada
	          "Canada/Central": '50.45,-104.62',
	          //Regina, SK, Canada
	          "Canada/East-Saskatchewan": '52.94,-106.45',
	          //Saskatchewan, Canada
	          "Canada/Eastern": '43.65,-79.38',
	          //Toronto, ON, Canada
	          "Canada/Mountain": '51.05,-114.07',
	          //Calgary, AB, Canada
	          "Canada/Newfoundland": '53.14,-57.66',
	          //Newfoundland and Labrador, Canada
	          "Canada/Pacific": '49.28,-123.12',
	          //Vancouver, BC, Canada
	          "Canada/Saskatchewan": '52.94,-106.45',
	          //Saskatchewan, Canada
	          "Canada/Yukon": '64.28,-135',
	          //Yukon Territory, Canada
	          "Chile/Continental": '-33.45,-70.67',
	          //Santiago, Santiago Metropolitan Region, Chile
	          "Chile/EasterIsland": '-27.11,-109.35',
	          //Easter Island, Valparaiso Region, Chile
	          "Europe/Belfast": '54.6,-5.93',
	          //Belfast, UK
	          "Europe/Busingen": '47.7,8.69',
	          //78266 Büsingen am Hochrhein, Germany
	          "Europe/Nicosia": '35.19,33.38',
	          //Nicosia, Cyprus
	          "Europe/Tiraspol": '46.85,29.6',
	          //Tiraspol, Moldova
	          "Mexico/BajaNorte": '30.84,-115.28',
	          //Baja California, Mexico
	          "Mexico/BajaSur": '26.04,-111.67',
	          //Baja California Sur, Mexico
	          "Mexico/General": '18.51,-96.47',
	          //General, Tetela, Oax., Mexico
	          "Pacific/Ponape": '6.97,158.2',
	          //Kolonia, Pohnpei 96941, Federated States of Micronesia
	          "Pacific/Samoa": '-13.76,-172.1',
	          //Samoa
	          "Pacific/Truk": '7.45,151.75',
	          //Chuuk Lagoon, Federated States of Micronesia
	          "Pacific/Yap": '33.79,-118.19',
	          //1939 Pacific Avenue, Long Beach, CA 90806, USA
	          //these are pretty arbitrary and I made them up myself
	          "Etc/GMT": '51.5076,-0.1276',
	          //London
	          "Etc/GMT+0": '51.5076,-0.1276',
	          //London
	          "Etc/GMT-0": '51.5076,-0.1276',
	          //London
	          "Etc/GMT+1": '48.8589,2.3469',
	          //Paris
	          "Etc/GMT+2": '47.4816,19.1300',
	          //budapest
	          "Etc/GMT+3": '33.3156,44.3867',
	          //baghdad
	          "Etc/GMT+4": '25.0749,55.4595',
	          //dubai
	          "Etc/GMT+5": '33.6940,73.0652',
	          //islamabad
	          "Etc/GMT+6": '23.7595,90.3788',
	          //dhaka
	          "Etc/GMT+7": '13.5862,100.6332',
	          //bangkok
	          "Etc/GMT+8": '39.9061,116.3912',
	          //beijing
	          "Etc/GMT+9": '28.345,145.030',
	          //tokyo
	          "Etc/GMT+10": '-27.3396,153.0881',
	          //brisbane
	          "Etc/GMT+11": '6.9248,158.1611',
	          //palakir
	          "Etc/GMT+12": '-36.8470,174.7727',
	          //aukland
	          "Etc/GMT-1": '14.6935,-17.4479',
	          //dakar
	          "Etc/GMT-2": '64.1755,-51.7355',
	          //nuuk
	          "Etc/GMT-3": '-22.662,-48.635',
	          //Sao Paulo
	          "Etc/GMT-4": '10.5061,-66.9146',
	          //caracas
	          "Etc/GMT-5": '4.5981,-74.0761',
	          //bogota
	          "Etc/GMT-6": '13.6990,-89.1914',
	          //san salvador
	          "Etc/GMT-7": '33.41329,-112.00902',
	          //pheonix AZ
	          "Etc/GMT-8": '61.2167,-149.8949',
	          //anchorage (way off)
	          "Etc/GMT-9": '-25.0667,-130.1002',
	          //adamstown (not technically in -9)
	          "Etc/GMT-10": '-17.5571,-149.5554',
	          //papeete
	          "Etc/GMT-11": '-19.0531,-169.8958',
	          //alofi
	          "Etc/GMT-12": '0.1954,-176.4786',
	          //baker island
	          "Etc/GMT-13": '6.9248,158.1611',
	          //palakir (+11)
	          "Etc/GMT-14": '-27.3396,153.0881',
	          //brisbane (+10)
	          "Etc/GMT0": '51.5076,-0.1276',
	          //London
	          "Etc/Greenwich": '51.5076,-0.1276',
	          //London
	          "Etc/UCT": '51.5076,-0.1276',
	          //London
	          "Etc/UTC": '51.5076,-0.1276',
	          //London
	          "Etc/Universal": '51.5076,-0.1276',
	          //London
	          "Etc/Zulu": '51.5076,-0.1276' //London

	        };
	      }, {}],
	      5: [function (_dereq_, module, exports) {

	        var points = _dereq_('./IANA-points'); //


	        var point = function point() {
	          var tz = this.timezone().name;

	          if (points.hasOwnProperty(tz) === false) {
	            console.warn('Unable to find location for timezone ' + tz);
	            return {};
	          }

	          var arr = points[tz].split(',');
	          return {
	            lat: parseFloat(arr[0]),
	            lng: parseFloat(arr[1])
	          };
	        };

	        module.exports = point;
	      }, {
	        "./IANA-points": 4
	      }]
	    }, {}, [3])(3);
	  });
	});

	function toDegree(radians) {
	  var pi = Math.PI;
	  return radians * (180 / pi);
	}

	var sunPosition = function sunPosition(s, lat, lng) {
	  if (lat === undefined || lng === undefined) {
	    var guess = s.point();
	    lat = guess.lat;
	    lng = guess.lng;
	  }

	  s["in"] = s["in"] || spacetimeGeo["in"]; //bolt-on the plugin

	  s = s["in"](lat, lng);
	  var d = new Date(s.epoch);
	  var res = suncalc.getPosition(d, lat, lng); // return res

	  return {
	    azimuth: toDegree(res.azimuth),
	    altitude: toDegree(res.altitude)
	  };
	};

	var sunPosition_1 = sunPosition;

	// the average time between solstices, on timeanddate.com
	// approx 88 days, 23 hours, 30 mins
	var oneYear = 31557060000;
	var halfYear = 15855660000; // strangely, this does not seem to be exactly half.
	// const halfYear = oneYear / 2
	// the 2015 winter solstice

	var oneWinter = 1450759620000;

	var goForward = function goForward(epoch) {
	  var num = oneWinter + oneYear;

	  while (num < epoch) {
	    num += oneYear;
	  }

	  return num;
	};

	var goBackward = function goBackward(epoch) {
	  var num = oneWinter - oneYear;

	  while (num > epoch) {
	    num -= oneYear;
	  }

	  return num;
	};

	var solstice = function solstice(s) {
	  var found = null;

	  if (s.epoch > oneWinter) {
	    found = goForward(s.epoch);
	  } else {
	    found = goBackward(s.epoch);
	  }

	  var winter = s.set(found); // ensure it's the right year

	  if (winter.year() < s.year()) {
	    winter = winter.set(winter.epoch + oneYear);
	  }

	  if (winter.year() > s.year()) {
	    winter = winter.set(winter.epoch - oneYear);
	  }

	  var summer = winter.set(winter.epoch - halfYear);
	  return {
	    winter: winter,
	    summer: summer
	  };
	}; // const equinox = function (s) {
	//   return {
	//     summer: null,
	//     winter: null,
	//   }
	// }


	var solstices = {
	  solstice: solstice
	};

	var _solstice = solstices.solstice; //equinox

	var setFrom = function setFrom(s, time) {
	  var d = new Date(time); // console.log(time)

	  s = s.clone();
	  s.epoch = d.getTime();
	  return s;
	};

	var calculatePoint = function calculatePoint(s, lat, lng, field) {
	  if (lat === undefined || lng === undefined) {
	    var guess = s.point();
	    lat = guess.lat;
	    lng = guess.lng;
	  }

	  s["in"] = s["in"] || spacetimeGeo["in"]; //bolt-on the plugin

	  s = s["in"](lat, lng);
	  var d = new Date(s.epoch);
	  var res = suncalc.getTimes(d, lat, lng);
	  return setFrom(s, res[field]);
	};

	var src = {
	  //depend on this plugin
	  "in": spacetimeGeo["in"],
	  point: spacetimeGeo.point,
	  solstice: function solstice() {
	    return _solstice(this);
	  },
	  winterSolstice: function winterSolstice() {
	    return _solstice(this).winter;
	  },
	  summerSolstice: function summerSolstice() {
	    return _solstice(this).summer;
	  },
	  sunPosition: function sunPosition(lat, lng) {
	    return sunPosition_1(this, lat, lng);
	  },
	  sunrise: function sunrise(lat, lng) {
	    return calculatePoint(this, lat, lng, 'sunrise');
	  },
	  sunset: function sunset(lat, lng) {
	    return calculatePoint(this, lat, lng, 'sunset');
	  },
	  noon: function noon(lat, lng) {
	    return calculatePoint(this, lat, lng, 'solarNoon');
	  },
	  dawn: function dawn(lat, lng) {
	    return calculatePoint(this, lat, lng, 'dawn');
	  },
	  dusk: function dusk(lat, lng) {
	    return calculatePoint(this, lat, lng, 'dusk');
	  },
	  daylight: function daylight(lat, lng) {
	    var sunrise = this.sunrise(lat, lng);
	    var sunset = this.sunset(lat, lng);
	    var delta = sunset.since(sunrise); //clean this up a bit

	    var duration = {
	      hours: delta.diff.hours,
	      minutes: delta.diff.minutes,
	      seconds: delta.diff.seconds
	    };
	    var diff = sunrise.diff(sunset);
	    diff.seconds = parseInt((sunset.epoch - sunrise.epoch) / 1000, 10);
	    var now = sunrise.diff(this);
	    now.seconds = parseInt((this.epoch - sunrise.epoch) / 1000, 10);
	    var progress = now.seconds / diff.seconds;
	    var status = 'day';
	    var dawn = this.dawn();
	    var dusk = this.dusk();

	    if (progress < 0) {
	      progress = 0;

	      if (this.epoch > dawn.epoch) {
	        status = 'dawn';
	      } else {
	        status = 'night';
	      }
	    } else if (progress > 1) {
	      progress = 0;

	      if (this.epoch < dusk.epoch) {
	        status = 'dusk';
	      } else {
	        status = 'night';
	      }
	    }

	    return {
	      dawn: dawn.time(),
	      sunrise: sunrise.time(),
	      sunset: sunset.time(),
	      dusk: dusk.time(),
	      duration: {
	        inHours: diff.hours,
	        inMinutes: diff.minutes,
	        inSeconds: diff.seconds,
	        human: duration
	      },
	      current: {
	        progress: progress,
	        status: status
	      }
	    };
	  }
	};
	var src_1 = src.point;
	var src_2 = src.solstice;
	var src_3 = src.winterSolstice;
	var src_4 = src.summerSolstice;
	var src_5 = src.sunPosition;
	var src_6 = src.sunrise;
	var src_7 = src.sunset;
	var src_8 = src.noon;
	var src_9 = src.dawn;
	var src_10 = src.dusk;
	var src_11 = src.daylight;

	exports.dawn = src_9;
	exports.daylight = src_11;
	exports.default = src;
	exports.dusk = src_10;
	exports.noon = src_8;
	exports.point = src_1;
	exports.solstice = src_2;
	exports.summerSolstice = src_4;
	exports.sunPosition = src_5;
	exports.sunrise = src_6;
	exports.sunset = src_7;
	exports.winterSolstice = src_3;

	Object.defineProperty(exports, '__esModule', { value: true });

})));
